#ifndef AVRootloader_H
#define AVRootloader_H

#include <avr/io.h>
#include <avr/wdt.h>
#include "intcast.h"

// start bootloader with direct jump
static inline void bootloader_start_jump(void) __attribute__((always_inline, noreturn));
// start bootloader trough watchdog reset
static inline void bootloader_start_wdt(void) __attribute__((always_inline, noreturn));
// retrieve size in bytes of bootmsg
static inline uint16_t bootmsg_size(void) __attribute__((always_inline));
// retrieve flash address of bootmsg
static inline uint32_t bootmsg_addr(void) __attribute__((always_inline));
// retrieve application version number, if not supported return 0xFFFFFFFF
static inline uint32_t get_appversion(void) __attribute__((always_inline));


#define GETAPPVERSION_VEC   FLASHEND -5
#define GETBOOTMSG_VEC      FLASHEND -3
#define BOOTSTART_VEC       FLASHEND -1


void bootloader_start_jump(void) {

    asm volatile("jmp -2");
}

void bootloader_start_wdt(void) {

    wdt_reset();
    _WD_CONTROL_REG = (1 << WDE) | (1 << _WD_CHANGE_BIT);
    _WD_CONTROL_REG = (1 << WDE);
    for (;;);
}

uint16_t bootmsg_size(void) {

    uint16_t (*proc)(void) = (void*)GETBOOTMSG_VEC;
    return H8(proc());
}

uint32_t bootmsg_addr(void) {

    uint32_t (*proc)(void) = (void*)GETBOOTMSG_VEC;
    uint32_t res = proc();
    asm volatile("clr %D0" : "+&r" (res) : "r" (res));
    return res;
}

uint32_t get_appversion(void) {

     uint32_t (*proc)(void) = (void*)GETAPPVERSION_VEC;
     return proc();
}

#endif
